var onReady = function(init) {
	document.addEventListener("DOMContentLoaded", init, false);
}

var __eventHandlers__ = {};

var isSet = function (arg) {
	return typeof arg !== 'undefined';
}

var addEvent = function (group, node, event, handler, capture) {
	if (!isSet(capture)) {
		capture = false;
	}
	
	var nodeId = node.id;
	
	if (!(group in __eventHandlers__)) {
		// _eventHandlers stores references to groups
		__eventHandlers__[group] = {};
	}
	if (!(nodeId in __eventHandlers__[group])) {
		// _eventHandlers stores references to nodes
		__eventHandlers__[group][nodeId] = {ref : node};
	}
	if (!(event in __eventHandlers__[group][nodeId])) {
		// each entry contains another entry for each event type
		__eventHandlers__[group][nodeId][event] = [];
	}
	// capture reference
	__eventHandlers__[group][nodeId][event].push([handler, capture]); //register handler
	
	node.addEventListener(event, handler, capture);
}

var removeEvent = function (group, node, event) {
	if (group in __eventHandlers__) {
		
		var nodes = __eventHandlers__[group];
		
		var nodesObj = {};
		
		// if node is set use it, otherwise use all nodes for the given group
		if (isSet(node)) {
			nodesObj[node.id] = nodes[node.id];
		} else {
			nodesObj = nodes;
		}
			
		for (var n in nodesObj) { 

			var eventsObj = {};
			
			// if event is set use it, otherwise use all events for the given node
			if (isSet(event)) {
				eventsObj[event] = nodesObj[n][event];
			} else {
				eventsObj = nodesObj[n];
			}
			
			for (var e in eventsObj) {
				if (e !== 'ref') {
					console.log(e);
				
					var eventHandlers = nodes[n][e];
					
					// iterate through all cached handlers of an event
					for (var h = eventHandlers.length; h--;) {
						var handler = eventHandlers[h];
						
						nodes[n]['ref'].removeEventListener(e, handler[0], handler[1]); // remove the event
					}
					
					// delete the event obj	
					delete nodes[n][e];
					
					// delete the node obj if empty (1 because of the ref attribute)
					if (Object.keys(nodes[n]).length === 1) {
						delete nodes[n];
					}
				}
			}
		}

		// delete the group obj if empty
		if (Object.keys(nodes).length === 0) {
			delete __eventHandlers__[group];
		}			
	}
}




Element.prototype.onTap = function(group, handler, useCapture) {
	var elem = this;
	var moved = false;
	var useCapture = useCapture || false;
	var t1, t2;
	
	var isTouchDevice = (function () {
		return "ontouchstart" in window // works on most browsers
			|| "onmsgesturechange" in window; // works on ie10
	})();
	
	if (!isTouchDevice) {
		addEvent(group, elem, "click", handler, useCapture);
		return;
	}
	
	addEvent(group, elem, "touchstart", function(e) {
		console.log('touchstart');
		t1 = new Date().getTime();
		
		if (moved) {
        	e.preventDefault();
        }
		elem.className = 'hover';
		
        moved = false;
	}, useCapture);
	
	addEvent(group, elem, "touchend", function(e) {
		console.log('touchend');
		
		t2 = new Date().getTime();

		if ((t2 - t1) > 200) {
			moved = true;
		}
		
		elem.className = '';
				
		e.preventDefault();
		
		if (!moved) {
			setTimeout(function() {
				handler(e, elem);
			}, 1);
		}
        moved = false;
	}, useCapture);
}


Element.prototype.load = function ( url, callback, notReady ) {	
	if (window.XMLHttpRequest) {
		var xmlhttp = new XMLHttpRequest(); // code for IE7+, Firefox, Chrome, Opera, Safari
	}
	else {
		var xmlhttp = new ActiveXObject( "Microsoft.XMLHTTP" ); // code for IE6, IE5
	}
	
	var that = this;
	
	xmlhttp.open('GET', url, true);
	xmlhttp.onreadystatechange = function() {
		if (xmlhttp.readyState != 4) {
			if (isSet(notReady))
				notReady();
		}
		if (xmlhttp.readyState == 4 && xmlhttp.status == 200) {
			that.innerHTML = xmlhttp.responseText;
			if (isSet(callback))
				callback();
		}
	}
	xmlhttp.send();
}